@@ -1,5 +1,5 @@
 package Net::Amazon::S3;
-$Net::Amazon::S3::VERSION = '0.88';
+$Net::Amazon::S3::VERSION = '0.90';
 use Moose 0.85;
 use MooseX::StrictConstructor 0.16;
 
@@ -8,7 +8,7 @@ use MooseX::StrictConstructor 0.16;
 
 use Carp;
 use Digest::HMAC_SHA1;
-use Scalar::Util;
+use Safe::Isa ();
 
 use Net::Amazon::S3::Bucket;
 use Net::Amazon::S3::Client;
@@ -32,10 +32,13 @@ use Net::Amazon::S3::Request::ListBucket;
 use Net::Amazon::S3::Request::ListParts;
 use Net::Amazon::S3::Request::PutObject;
 use Net::Amazon::S3::Request::PutPart;
+use Net::Amazon::S3::Request::RestoreObject;
 use Net::Amazon::S3::Request::SetBucketAccessControl;
 use Net::Amazon::S3::Request::SetObjectAccessControl;
 use Net::Amazon::S3::Signature::V2;
 use Net::Amazon::S3::Signature::V4;
+use Net::Amazon::S3::Vendor;
+use Net::Amazon::S3::Vendor::Amazon;
 use LWP::UserAgent::Determined;
 use URI::Escape qw(uri_escape_utf8);
 use XML::LibXML;
@@ -43,52 +46,110 @@ use XML::LibXML::XPathContext;
 
 my $AMAZON_S3_HOST = 's3.amazonaws.com';
 
-has 'use_iam_role' => ( is => 'ro', isa => 'Bool', required => 0, default => 0);
-has 'aws_access_key_id'     => ( is => 'rw', isa => 'Str', required => 0 );
-has 'aws_secret_access_key' => ( is => 'rw', isa => 'Str', required => 0 );
-has 'secure' => ( is => 'ro', isa => 'Bool', required => 0, default => 1 );
+has authorization_context => (
+	is => 'ro',
+	isa => 'Net::Amazon::S3::Authorization',
+	required => 0,
+
+	handles => {
+		aws_access_key_id     => 'aws_access_key_id',
+		aws_secret_access_key => 'aws_secret_access_key',
+		aws_session_token     => 'aws_session_token',
+	},
+);
+
+has vendor => (
+	is => 'ro',
+	isa => 'Net::Amazon::S3::Vendor',
+	required => 1,
+
+	handles => {
+		authorization_method => 'authorization_method',
+		host                 => 'host',
+		secure               => 'use_https',
+		use_virtual_host     => 'use_virtual_host',
+	},
+);
+
 has 'timeout' => ( is => 'ro', isa => 'Num',  required => 0, default => 30 );
 has 'retry'   => ( is => 'ro', isa => 'Bool', required => 0, default => 0 );
-has 'host'    => ( is => 'ro', isa => 'Str',  required => 0, default => $AMAZON_S3_HOST );
-has 'use_virtual_host' => (
-    is => 'ro',
-    isa => 'Bool',
-    required => 0,
-    lazy => 1,
-    default => sub { $_[0]->authorization_method->enforce_use_virtual_host },
-);
 has 'libxml' => ( is => 'rw', isa => 'XML::LibXML',    required => 0 );
 has 'ua'     => ( is => 'rw', isa => 'LWP::UserAgent', required => 0 );
 has 'err'    => ( is => 'rw', isa => 'Maybe[Str]',     required => 0 );
 has 'errstr' => ( is => 'rw', isa => 'Maybe[Str]',     required => 0 );
-has 'aws_session_token' => ( is => 'rw', isa => 'Str', required => 0 );
-has authorization_method => (
-    is => 'ro',
-    isa => 'Str',
-    required => 0,
-    lazy => 1,
-    default => sub {
-        $_[0]->host eq $AMAZON_S3_HOST
-            ? 'Net::Amazon::S3::Signature::V4'
-            : 'Net::Amazon::S3::Signature::V2'
-    },
-);
-
 has keep_alive_cache_size => ( is => 'ro', isa => 'Int', required => 0, default => 10 );
 
-__PACKAGE__->meta->make_immutable;
+
+sub _build_arg_authorization_context {
+	my ($args) = @_;
+
+	my $aws_access_key_id     = delete $args->{aws_access_key_id};
+	my $aws_secret_access_key = delete $args->{aws_secret_access_key};
+	my $use_iam_role          = delete $args->{use_iam_role};
+	my $aws_session_token     = delete $args->{aws_session_token};
+
+	if ($args->{authorization_context}) {
+		return $args->{authorization_context};
+	}
+
+	if ($use_iam_role || $aws_session_token) {
+		require Net::Amazon::S3::Authorization::IAM;
+
+		return Net::Amazon::S3::Authorization::IAM->new (
+			aws_access_key_id     => $aws_access_key_id,
+			aws_secret_access_key => $aws_secret_access_key,
+			aws_session_token     => $aws_session_token,
+		)
+	}
+
+	require Net::Amazon::S3::Authorization::Basic;
+
+	return Net::Amazon::S3::Authorization::Basic->new (
+		aws_access_key_id => $aws_access_key_id,
+		aws_secret_access_key => $aws_secret_access_key,
+	);
+}
+
+sub _build_arg_vendor {
+	my ($args) = @_;
+
+	my %backward =
+		map  { $_ => delete $args->{$_} }
+		grep { exists  $args->{$_} }
+		qw[ host secure use_virtual_host authorization_method ]
+		;
+
+	return $args->{vendor}
+		if $args->{vendor};
+
+	$backward{host} = $AMAZON_S3_HOST
+		unless exists $backward{host};
+
+	$backward{use_https} = delete $backward{secure}
+		if exists $backward{secure};
+
+	my $vendor_class = $backward{host} eq $AMAZON_S3_HOST
+		? 'Net::Amazon::S3::Vendor::Amazon'
+		: 'Net::Amazon::S3::Vendor'
+		;
+
+	return $vendor_class->new (%backward);
+}
+
+around BUILDARGS => sub {
+	my ($orig, $class, %args) = @_;
+
+	# support compat authorization arguments
+	$args{authorization_context} = _build_arg_authorization_context \%args;
+	$args{vendor}                = _build_arg_vendor                \%args;
+
+    $class->$orig (%args);
+};
 
 
 sub BUILD {
     my $self = shift;
 
-    if (!$self->use_iam_role) {
-        if (!defined($self->aws_secret_access_key) || !defined($self->aws_access_key_id)) {
-            die("Must specify aws_secret_access_key and aws_access_key_id");
-        }
-    }
-
-
     my $ua;
     if ( $self->retry ) {
         $ua = LWP::UserAgent::Determined->new(
@@ -108,15 +169,6 @@ sub BUILD {
 
     $self->ua($ua);
     $self->libxml( XML::LibXML->new );
-
-    if ($self->use_iam_role) {
-        eval "require VM::EC2::Security::CredentialCache" or die $@;
-        my $creds = VM::EC2::Security::CredentialCache->get();
-        defined($creds) || die("Unable to retrieve IAM role credentials");
-        $self->aws_access_key_id($creds->accessKeyId);
-        $self->aws_secret_access_key($creds->secretAccessKey);
-        $self->aws_session_token($creds->sessionToken);
-    }
 }
 
 
@@ -164,6 +216,7 @@ sub add_bucket {
         bucket              => $conf->{bucket},
         acl_short           => $conf->{acl_short},
         location_constraint => $conf->{location_constraint},
+        ( $conf->{region} ? (region => $conf->{region}) : () ),
     )->http_request;
 
     return 0
@@ -176,8 +229,7 @@ sub add_bucket {
 sub bucket {
     my ( $self, $bucket ) = @_;
 
-    return $bucket
-        if Scalar::Util::blessed( $bucket ) && $bucket->isa( 'Net::Amazon::S3::Bucket' );
+    return $bucket if $bucket->$Safe::Isa::_isa ('Net::Amazon::S3::Bucket');
 
     return Net::Amazon::S3::Bucket->new(
         { bucket => $bucket, account => $self } );
@@ -443,6 +495,8 @@ sub _urlencode {
     return uri_escape_utf8( $unencoded, '^A-Za-z0-9_\-\.' );
 }
 
+__PACKAGE__->meta->make_immutable;
+
 1;
 
 __END__
@@ -457,22 +511,31 @@ Net::Amazon::S3 - Use the Amazon S3 - Simple Storage Service
 
 =head1 VERSION
 
-version 0.88
+version 0.90
 
 =head1 SYNOPSIS
 
   use Net::Amazon::S3;
+  use Net::Amazon::S3::Authorization::Basic;
+  use Net::Amazon::S3::Authorization::IAM;
   my $aws_access_key_id     = 'fill me in';
   my $aws_secret_access_key = 'fill me in too';
 
-  my $s3 = Net::Amazon::S3->new(
-      {   aws_access_key_id     => $aws_access_key_id,
-          aws_secret_access_key => $aws_secret_access_key,
-          # or use an IAM role.
-          use_iam_role          => 1
+  my $s3 = Net::Amazon::S3->new (
+    authorization_context => Net::Amazon::S3::Authorization::Basic->new (
+      aws_access_key_id     => $aws_access_key_id,
+      aws_secret_access_key => $aws_secret_access_key,
+    ),
+    retry => 1,
+  );
 
-          retry                 => 1,
-      }
+  # or use an IAM role.
+  my $s3 = Net::Amazon::S3->new (
+    authorization_context => Net::Amazon::S3::Authorization::IAM->new (
+      aws_access_key_id     => $aws_access_key_id,
+      aws_secret_access_key => $aws_secret_access_key,
+    ),
+    retry => 1,
   );
 
   # a bucket is a globally-unique directory
@@ -565,8 +628,47 @@ Create a new S3 client object. Takes some arguments:
 
 =over
 
+=item authorization_context
+
+Class that provides authorization informations.
+
+See one of available implementations for more
+
+=over
+
+=item L<Net::Amazon::S3::Authorization::Basic>
+
+=item L<Net::Amazon::S3::Authorization::IAM>
+
+=back
+
+=item vendor
+
+Instance of L<Net::Amazon::S3::Vendor> holding vendor specific deviations.
+
+S3 became widely used object storage protocol with many vendors providing
+different feature sets and different compatibility level.
+
+One common difference is bucket's HEAD request to determine its region.
+
+To maintain currently known differences along with any differencies that
+may rise in feature it's better to hold vendor specification in dedicated
+classes. This also allows users to build their own fine-tuned vendor classes.
+
+=over
+
+=item L<Net::Amazon::S3::Vendor::Amazon>
+
+=item L<Net::Amazon::S3::Vendor::Generic>
+
+=back
+
 =item aws_access_key_id
 
+Deprecated.
+
+When used it's used to create authorization context.
+
 Use your Access Key ID as the value of the AWSAccessKeyId parameter
 in requests you send to Amazon Web Services (when required). Your
 Access Key ID identifies you as the party responsible for the
@@ -574,6 +676,10 @@ request.
 
 =item aws_secret_access_key
 
+Deprecated.
+
+When used it's used to create authorization context.
+
 Since your Access Key ID is not encrypted in requests to AWS, it
 could be discovered and used by anyone. Services that are not free
 require you to provide additional information, a request signature,
@@ -584,22 +690,34 @@ DO NOT INCLUDE THIS IN SCRIPTS OR APPLICATIONS YOU DISTRIBUTE. YOU'LL BE SORRY
 
 =item aws_session_token
 
+Deprecated.
+
+When used it's used to create authorization context.
+
 If you are using temporary credentials provided by the AWS Security Token
 Service, set the token here, and it will be added to the request in order to
 authenticate it.
 
 =item use_iam_role
 
+Deprecated.
+
+When used it's used to create authorization context.
+
 If you'd like to use IAM provided temporary credentials, pass this option
 with a true value.
 
 =item secure
 
+Deprecated.
+
 Set this to C<0> if you don't want to use SSL-encrypted connections when talking
 to S3. Defaults to C<1>.
 
 To use SSL-encrypted connections, LWP::Protocol::https is required.
 
+See L<#vendor> and L<Net::Amazon::S3::Vendor>.
+
 =item keep_alive_cache_size
 
 Set this to C<0> to disable Keep-Alives.  Default is C<10>.
@@ -617,24 +735,36 @@ as recommended by Amazon. Defaults to off.
 
 =item host
 
+Deprecated.
+
 The S3 host endpoint to use. Defaults to 's3.amazonaws.com'. This allows
 you to connect to any S3-compatible host.
 
+See L<#vendor> and L<Net::Amazon::S3::Vendor>.
+
 =item use_virtual_host
 
+Deprecated.
+
 Use the virtual host method ('bucketname.s3.amazonaws.com') instead of specifying the
 bucket at the first part of the path. This is particularly useful if you want to access
 buckets not located in the US-Standard region (such as EU, Asia Pacific or South America).
 See L<http://docs.aws.amazon.com/AmazonS3/latest/dev/VirtualHosting.html> for the pros and cons.
 
+See L<#vendor> and L<Net::Amazon::S3::Vendor>.
+
 =item authorization_method
 
+Deprecated.
+
 Authorization implementation package name.
 
 This library provides L<< Net::Amazon::S3::Signature::V2 >> and L<< Net::Amazon::S3::Signature::V4 >>
 
 Default is Signature 4 if host is C<< s3.amazonaws.com >>, Signature 2 otherwise
 
+See L<#vendor> and L<Net::Amazon::S3::Vendor>.
+
 =back
 
 =head3 Notes
