@@ -1,32 +1,33 @@
 package Net::Amazon::S3::Role::Bucket;
 # ABSTRACT: Bucket role
-$Net::Amazon::S3::Role::Bucket::VERSION = '0.89';
+$Net::Amazon::S3::Role::Bucket::VERSION = '0.90';
 use Moose::Role;
-use Scalar::Util;
+use Safe::Isa ();
 
 around BUILDARGS => sub {
     my ($orig, $class, %params) = @_;
 
-    $params{region} = $params{bucket}->region
-        if $params{bucket}
-        and Scalar::Util::blessed( $params{bucket} )
-        and ! $params{region}
-        and $params{bucket}->has_region
-        ;
+	# bucket can be optional in HTTPRequest
+	if ($params{bucket}) {
+		my $region = $params{region};
+		$region = $params{bucket}->region
+			if $params{bucket}
+			and Scalar::Util::blessed( $params{bucket} )
+			and ! $params{region}
+			and $params{bucket}->has_region
+			;
 
-    $params{bucket} = $params{bucket}->name
-        if $params{bucket}
-        and Scalar::Util::blessed( $params{bucket} )
-        and $params{bucket}->isa( 'Net::Amazon::S3::Client::Bucket' )
-        ;
+		$params{bucket} = $params{bucket}->name
+			if $params{bucket}->$Safe::Isa::_isa ('Net::Amazon::S3::Client::Bucket');
 
-    $params{bucket} = Net::Amazon::S3::Bucket->new(
-        bucket => $params{bucket},
-        account => $params{s3},
-        (region => $params{region}) x!! $params{region},
-    ) if $params{bucket} and ! ref $params{bucket};
+		$params{bucket} = Net::Amazon::S3::Bucket->new(
+			bucket => $params{bucket},
+			account => $params{s3},
+			(region => $region) x!! $region,
+		) if $params{bucket} and ! ref $params{bucket};
 
-    delete $params{region};
+		delete $params{region};
+	}
 
     $class->$orig( %params );
 };
@@ -51,7 +52,7 @@ Net::Amazon::S3::Role::Bucket - Bucket role
 
 =head1 VERSION
 
-version 0.89
+version 0.90
 
 =head1 AUTHOR
 
