@@ -1,6 +1,6 @@
 package Shared::Examples::Net::Amazon::S3::API;
 # ABSTRACT: used for testing and as example
-$Shared::Examples::Net::Amazon::S3::API::VERSION = '0.88';
+$Shared::Examples::Net::Amazon::S3::API::VERSION = '0.90';
 use strict;
 use warnings;
 
@@ -28,8 +28,11 @@ our @EXPORT_OK = (
     qw[ expect_api_object_delete ],
     qw[ expect_api_object_fetch ],
     qw[ expect_api_object_head ],
+    qw[ with_fixture ],
 );
 
+*with_fixture = *Shared::Examples::Net::Amazon::S3::with_fixture;
+
 sub _exporter_expand_sub {
     my ($self, $name, $args, $globals) = @_;
 
@@ -51,29 +54,12 @@ sub _default_with_api {
 }
 
 sub _mock_http_response {
-    my (undef, %params) = @_;
+    my (undef, $api, %params) = @_;
 
-    $params{with_response_code} ||= HTTP::Status::HTTP_OK;
-
-    my %headers = (
-        content_type => 'application/xml',
-        %{ $params{with_response_headers} || {} },
-    );
-
-    my $guard = Sub::Override->new;
-    $guard->replace (
-        'Net::Amazon::S3::_do_http' => sub {
-            ${ $params{into} } = $_[1];
-            HTTP::Response->new (
-                $params{with_response_code},
-                HTTP::Status::status_message ($params{with_response_code}),
-                [ %headers ],
-                $params{with_response_data},
-            ),
-        }
-    );
-
-    $guard;
+    Shared::Examples::Net::Amazon::S3->s3_api_mock_http_response (
+        $api,
+        %params,
+    )
 }
 
 sub expect_signed_uri {
@@ -139,6 +125,7 @@ sub operation_bucket_create {
             bucket => $params{with_bucket},
             (acl_short => $params{with_acl}) x!! exists $params{with_acl},
             (location_constraint => $params{with_region}) x!! exists $params{with_region},
+            (region => $params{with_region}) x!! exists $params{with_region},
         })
         ;
 }
@@ -270,7 +257,7 @@ Shared::Examples::Net::Amazon::S3::API - used for testing and as example
 
 =head1 VERSION
 
-version 0.88
+version 0.90
 
 =head1 AUTHOR
 
